import requests
import os
import tqdm
import logging
from pydantic import BaseModel
from typing import List, Dict, Any

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

if not logger.handlers:
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.INFO)

    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
    console_handler.setFormatter(formatter)

    logger.addHandler(console_handler)

def download_file(url: str, dest_path: str, show_progress_bars: bool = True):
    """downloads a file and shows a progress bar. allow resuming a download"""
    file_size = 0
    req = requests.get(url, stream=True, timeout=600)
    req.raise_for_status()

    total_size = int(req.headers.get('content-length', 0))
    temp_path = dest_path + ".temp"

    if os.path.exists(dest_path):
        logger.info(f" file already exists: {dest_path}")
        file_size = os.stat(dest_path).st_size
        if file_size == total_size:
            return dest_path

    if os.path.exists(temp_path):
        file_size = os.stat(temp_path).st_size

        if file_size < total_size:
            # Download incomplete
            logger.info(f" resuming download")
            resume_header = {'Range': f'bytes={file_size}-'}
            req = requests.get(url, headers=resume_header, stream=True,
                               verify=False, allow_redirects=True, timeout=600)
        else:
            # Error, delete file and restart download
            logger.error(f" deleting file {dest_path} and restarting")
            os.remove(temp_path)
            file_size = 0
    else:
        # File does not exist, starting download
        logger.info(f" starting download")

    # write dataset to file and show progress bar
    pbar = tqdm.tqdm(total=total_size, unit='B', unit_scale=True,
                     desc=dest_path, disable=not show_progress_bars)
    # Update progress bar to reflect how much of the file is already downloaded
    pbar.update(file_size)
    with open(temp_path, "ab") as dest_file:
        for chunk in req.iter_content(1024):
            dest_file.write(chunk)
            pbar.update(1024)
    # move temp file to target destination
    os.replace(temp_path, dest_path)
    return dest_path


class SingleModel(BaseModel):
    coin_id: int
    evaluations: List[Dict[str, Any]]
    created: str
    model_id: int
    name: str
    status: str
    target: str
    updated: str
    dev_id: str
    target_type: str
    class Config:
        protected_namespaces = ()
    
    
class AllModels(BaseModel):
    models: List[SingleModel]


class AccountInfo(BaseModel):
    api_key: bool
    models: List[SingleModel]
    user_id: str
    username: str
    updated: str
    created: str


class ModelEvaluation(BaseModel):
    benchmarks: Dict[str, Dict[str, Any]]
    metrics: Dict[str, Any]


class ApiKeyGeneration(BaseModel):
    api_key: str


class DataInfo(BaseModel):
    data: Dict[str, Dict[str, Dict[str, List[str]]]]
    coins: List[str]
    feature_sizes: List[str]
    targets: Dict[str, str]
    versions: Dict[str, float]



from typing import Any
import pandas as pd
import requests
import os
from .utils import download_file, SingleModel, ModelEvaluation, DataInfo


class Crypticorn:
    """
    A client for interacting with the crypticorn API, offering functionality to create and evaluate models,
    download data, and retrieve information about available coins, targets, and features.
    """

    def __init__(self, api_key: str, headers: dict = None, base_url='https://api.crypticorn.com'):
        """@private
        Initializes the crypticorn API client with an API key.

        :param api_key: The API key required for authenticating requests.
        """
        self._base_url = base_url + "/v1/hive"
        self._headers = headers if headers else {"Authorization": f"ApiKey {api_key}"}

    def create_model(self, coin_id: int, target: str) -> SingleModel:
        """
        Creates a new model based on the specified coin_id and target.

        :param coin_id: The id of the coin to be used for the model.
        :param target: The target variable for the model.
        """
        endpoint = "/model/creation"
        response = requests.post(
            url=self._base_url + endpoint,
            params={"coin_id": coin_id, "target": target},
            headers=self._headers
        )
        return response.json()

    def evaluate_model(self, model_id: int, data: Any, version: float = None) -> ModelEvaluation:
        """
        Evaluates an existing model using the provided data.

        :param model_id: The id of the model to evaluate.
        :param data: The data to use for evaluation, which can be a pandas DataFrame or a file path with
                 extensions `.feather` or `.parquet`.
        :param version: (optional) Specifies the data version for evaluation. Defaults to the latest version.
                If a different version than the latest is specified, the evaluation will not be stored
                or counted on the leaderboard. This is useful for testing your model with different versions.
                Ensure to specify a `version` if your model was trained on older data versions; otherwise,
                it will be evaluated against the latest data, potentially affecting the results.
        """
        if isinstance(data, pd.DataFrame):
            json_data = data.to_json(orient='records')
        elif isinstance(data, str):
            if data.endswith('.feather'):
                json_data = pd.read_feather(data).to_json(orient="records")
            elif data.endswith('.parquet'):
                json_data = pd.read_parquet(data).to_json(orient="records")
            else:
                raise ValueError("Unsupported file format. Use .feather, .parquet, or pd.Dataframe.")
        else:
            raise ValueError("Unsupported data format. Pass a pd.DataFrame or a valid file path.")

        endpoint = "/model/evaluation"
        response = requests.post(
            url=self._base_url + endpoint,
            params={"model_id": model_id, "version": version},
            json=json_data,
            headers=self._headers
        )
        return response.json()

    def download_data(self, model_id: int, version: float = None,
                      feature_size: str = None) -> int:
        """
        Downloads training data for models.
        Either pass a model_id or coin_id. For more details about available data, use `data_info()`.

        :param model_id: id of the model to download data for.
        :param version: (optional) Data version to download. Defaults to the latest version if not specified.
        :param feature_size: (optional) Size of the feature set to download. Default is "large".
        """
        endpoint = "/data"
        response = requests.get(
            url=self._base_url + endpoint,
            params={"feature_size": feature_size, "version": version, "model_id": model_id},
            headers=self._headers
        )
        if response.status_code != 200:
            return response.json()
        data = response.json()
        base_path = f"v{data['version']}/coin_{data['coin']}/"
        os.makedirs(base_path, exist_ok=True)
        download_file(url=data["y_train"], dest_path=f"{base_path}y_train_{data['target']}.feather")
        download_file(url=data["X_test"], dest_path=f"{base_path}X_test_{data['feature_size']}.feather")
        download_file(url=data["X_train"], dest_path=f"{base_path}X_train_{data['feature_size']}.feather")
        return 200

    def data_info(self) -> DataInfo:
        """
        Returns information about the training data.
        Useful in combination with `download_data()` and `create_model()`.
        """
        endpoint = "/data/info"
        response = requests.get(
            url=self._base_url + endpoint,
            headers=self._headers)
        return response.json()



# coding: utf-8

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ApiErrorIdentifier(str, Enum):
    """
    API error identifiers
    """

    """
    allowed enum values
    """
    SUCCESS = 'success'
    INVALID_API_KEY = 'invalid_api_key'
    INVALID_SIGNATURE = 'invalid_signature'
    INVALID_TIMESTAMP = 'invalid_timestamp'
    IP_ADDRESS_IS_NOT_AUTHORIZED = 'ip_address_is_not_authorized'
    INSUFFICIENT_PERMISSIONS_SPOT_AND_FUTURES_REQUIRED = 'insufficient_permissions_spot_and_futures_required'
    USER_ACCOUNT_IS_FROZEN = 'user_account_is_frozen'
    RATE_LIMIT_EXCEEDED = 'rate_limit_exceeded'
    INVALID_PARAMETER_PROVIDED = 'invalid_parameter_provided'
    REQUEST_SCOPE_LIMIT_EXCEEDED = 'request_scope_limit_exceeded'
    INVALID_CONTENT_TYPE = 'invalid_content_type'
    REQUESTED_RESOURCE_NOT_FOUND = 'requested_resource_not_found'
    ORDER_DOES_NOT_EXIST = 'order_does_not_exist'
    ORDER_IS_ALREADY_FILLED = 'order_is_already_filled'
    ORDER_IS_BEING_PROCESSED = 'order_is_being_processed'
    ORDER_QUANTITY_LIMIT_EXCEEDED = 'order_quantity_limit_exceeded'
    ORDER_PRICE_IS_INVALID = 'order_price_is_invalid'
    POST_ONLY_ORDER_WOULD_IMMEDIATELY_MATCH = 'post_only_order_would_immediately_match'
    SYMBOL_DOES_NOT_EXIST = 'symbol_does_not_exist'
    POSITION_DOES_NOT_EXIST = 'position_does_not_exist'
    POSITION_LIMIT_EXCEEDED = 'position_limit_exceeded'
    POSITION_OPENING_TEMPORARILY_SUSPENDED = 'position_opening_temporarily_suspended'
    INSUFFICIENT_BALANCE = 'insufficient_balance'
    INSUFFICIENT_MARGIN = 'insufficient_margin'
    LEVERAGE_LIMIT_EXCEEDED = 'leverage_limit_exceeded'
    RISK_LIMIT_EXCEEDED = 'risk_limit_exceeded'
    ORDER_VIOLATES_LIQUIDATION_PRICE_CONSTRAINTS = 'order_violates_liquidation_price_constraints'
    INVALID_MARGIN_MODE = 'invalid_margin_mode'
    INTERNAL_SYSTEM_ERROR = 'internal_system_error'
    SYSTEM_CONFIGURATION_ERROR = 'system_configuration_error'
    SERVICE_TEMPORARILY_UNAVAILABLE = 'service_temporarily_unavailable'
    SYSTEM_IS_BUSY_PLEASE_TRY_AGAIN_LATER = 'system_is_busy_please_try_again_later'
    SYSTEM_UNDER_MAINTENANCE = 'system_under_maintenance'
    RPC_TIMEOUT = 'rpc_timeout'
    SYSTEM_SETTLEMENT_IN_PROCESS = 'system_settlement_in_process'
    TRADING_IS_SUSPENDED = 'trading_is_suspended'
    TRADING_HAS_BEEN_LOCKED = 'trading_has_been_locked'
    UNKNOWN_ERROR_OCCURRED = 'unknown_error_occurred'
    HTTP_REQUEST_ERROR = 'http_request_error'
    BLACK_SWAN = 'black_swan'
    TRADING_ACTION_EXPIRED = 'trading_action_expired'
    BOT_DISABLED = 'bot_disabled'
    NEW_TRADING_ACTION = 'new_trading_action'
    ORDER_SIZE_TOO_SMALL = 'order_size_too_small'
    ORDER_SIZE_TOO_LARGE = 'order_size_too_large'
    HEDGE_MODE_NOT_ACTIVE = 'hedge_mode_not_active'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ApiErrorIdentifier from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from crypticorn.models.api_error_identifier import ApiErrorIdentifier
from crypticorn.models.exchange import Exchange
from crypticorn.models.margin_mode import MarginMode
from crypticorn.models.market_type import MarketType
from crypticorn.models.order_status import OrderStatus
from crypticorn.models.trading_action_type import TradingActionType
from typing import Optional, Set
from typing_extensions import Self

class OrderModel(BaseModel):
    """
    Response model for orders. All optional as the model is built step by step.
    """ # noqa: E501
    id: Optional[StrictStr] = None
    trading_action_id: Optional[StrictStr] = None
    execution_id: Optional[StrictStr] = None
    exchange_order_id: Optional[StrictStr] = None
    position_id: Optional[StrictStr] = None
    api_key_id: Optional[StrictStr] = None
    user_id: Optional[StrictStr] = None
    bot_id: Optional[StrictStr] = None
    client_order_id: Optional[StrictStr] = None
    exchange: Optional[Exchange] = None
    symbol: Optional[StrictStr] = None
    common_symbol: Optional[StrictStr] = None
    timestamp: Optional[StrictInt] = None
    price: Optional[Union[StrictFloat, StrictInt]] = None
    action_type: Optional[TradingActionType] = None
    market_type: Optional[MarketType] = None
    margin_mode: Optional[MarginMode] = None
    status_code: Optional[ApiErrorIdentifier] = None
    status: Optional[OrderStatus] = None
    filled_perc: Optional[Union[StrictFloat, StrictInt]] = None
    filled_qty: Optional[Union[StrictFloat, StrictInt]] = None
    fee: Optional[Union[StrictFloat, StrictInt]] = None
    leverage: Optional[Union[StrictFloat, StrictInt]] = None
    order_details: Optional[Dict[str, Any]] = None
    pnl: Optional[Union[StrictFloat, StrictInt]] = None
    __properties: ClassVar[List[str]] = ["id", "trading_action_id", "execution_id", "exchange_order_id", "position_id", "api_key_id", "user_id", "bot_id", "client_order_id", "exchange", "symbol", "common_symbol", "timestamp", "price", "action_type", "market_type", "margin_mode", "status_code", "status", "filled_perc", "filled_qty", "fee", "leverage", "order_details", "pnl"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of OrderModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['id'] = None

        # set to None if trading_action_id (nullable) is None
        # and model_fields_set contains the field
        if self.trading_action_id is None and "trading_action_id" in self.model_fields_set:
            _dict['trading_action_id'] = None

        # set to None if execution_id (nullable) is None
        # and model_fields_set contains the field
        if self.execution_id is None and "execution_id" in self.model_fields_set:
            _dict['execution_id'] = None

        # set to None if exchange_order_id (nullable) is None
        # and model_fields_set contains the field
        if self.exchange_order_id is None and "exchange_order_id" in self.model_fields_set:
            _dict['exchange_order_id'] = None

        # set to None if position_id (nullable) is None
        # and model_fields_set contains the field
        if self.position_id is None and "position_id" in self.model_fields_set:
            _dict['position_id'] = None

        # set to None if api_key_id (nullable) is None
        # and model_fields_set contains the field
        if self.api_key_id is None and "api_key_id" in self.model_fields_set:
            _dict['api_key_id'] = None

        # set to None if user_id (nullable) is None
        # and model_fields_set contains the field
        if self.user_id is None and "user_id" in self.model_fields_set:
            _dict['user_id'] = None

        # set to None if bot_id (nullable) is None
        # and model_fields_set contains the field
        if self.bot_id is None and "bot_id" in self.model_fields_set:
            _dict['bot_id'] = None

        # set to None if client_order_id (nullable) is None
        # and model_fields_set contains the field
        if self.client_order_id is None and "client_order_id" in self.model_fields_set:
            _dict['client_order_id'] = None

        # set to None if exchange (nullable) is None
        # and model_fields_set contains the field
        if self.exchange is None and "exchange" in self.model_fields_set:
            _dict['exchange'] = None

        # set to None if symbol (nullable) is None
        # and model_fields_set contains the field
        if self.symbol is None and "symbol" in self.model_fields_set:
            _dict['symbol'] = None

        # set to None if common_symbol (nullable) is None
        # and model_fields_set contains the field
        if self.common_symbol is None and "common_symbol" in self.model_fields_set:
            _dict['common_symbol'] = None

        # set to None if timestamp (nullable) is None
        # and model_fields_set contains the field
        if self.timestamp is None and "timestamp" in self.model_fields_set:
            _dict['timestamp'] = None

        # set to None if price (nullable) is None
        # and model_fields_set contains the field
        if self.price is None and "price" in self.model_fields_set:
            _dict['price'] = None

        # set to None if action_type (nullable) is None
        # and model_fields_set contains the field
        if self.action_type is None and "action_type" in self.model_fields_set:
            _dict['action_type'] = None

        # set to None if market_type (nullable) is None
        # and model_fields_set contains the field
        if self.market_type is None and "market_type" in self.model_fields_set:
            _dict['market_type'] = None

        # set to None if margin_mode (nullable) is None
        # and model_fields_set contains the field
        if self.margin_mode is None and "margin_mode" in self.model_fields_set:
            _dict['margin_mode'] = None

        # set to None if status_code (nullable) is None
        # and model_fields_set contains the field
        if self.status_code is None and "status_code" in self.model_fields_set:
            _dict['status_code'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['status'] = None

        # set to None if filled_perc (nullable) is None
        # and model_fields_set contains the field
        if self.filled_perc is None and "filled_perc" in self.model_fields_set:
            _dict['filled_perc'] = None

        # set to None if filled_qty (nullable) is None
        # and model_fields_set contains the field
        if self.filled_qty is None and "filled_qty" in self.model_fields_set:
            _dict['filled_qty'] = None

        # set to None if fee (nullable) is None
        # and model_fields_set contains the field
        if self.fee is None and "fee" in self.model_fields_set:
            _dict['fee'] = None

        # set to None if leverage (nullable) is None
        # and model_fields_set contains the field
        if self.leverage is None and "leverage" in self.model_fields_set:
            _dict['leverage'] = None

        # set to None if order_details (nullable) is None
        # and model_fields_set contains the field
        if self.order_details is None and "order_details" in self.model_fields_set:
            _dict['order_details'] = None

        # set to None if pnl (nullable) is None
        # and model_fields_set contains the field
        if self.pnl is None and "pnl" in self.model_fields_set:
            _dict['pnl'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of OrderModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "trading_action_id": obj.get("trading_action_id"),
            "execution_id": obj.get("execution_id"),
            "exchange_order_id": obj.get("exchange_order_id"),
            "position_id": obj.get("position_id"),
            "api_key_id": obj.get("api_key_id"),
            "user_id": obj.get("user_id"),
            "bot_id": obj.get("bot_id"),
            "client_order_id": obj.get("client_order_id"),
            "exchange": obj.get("exchange"),
            "symbol": obj.get("symbol"),
            "common_symbol": obj.get("common_symbol"),
            "timestamp": obj.get("timestamp"),
            "price": obj.get("price"),
            "action_type": obj.get("action_type"),
            "market_type": obj.get("market_type"),
            "margin_mode": obj.get("margin_mode"),
            "status_code": obj.get("status_code"),
            "status": obj.get("status"),
            "filled_perc": obj.get("filled_perc"),
            "filled_qty": obj.get("filled_qty"),
            "fee": obj.get("fee"),
            "leverage": obj.get("leverage"),
            "order_details": obj.get("order_details"),
            "pnl": obj.get("pnl")
        })
        return _obj





from . import *
from .__init__ import *
from .action_model import *
from .api_error_identifier import *
from .api_key_model import *
from .bot_model import *
from .create_api_key_response import *
from .deleted import *
from .exchange import *
from .execution_ids import *
from .futures_balance import *
from .futures_balance_error import *
from .futures_trading_action import *
from .get_futures_balance200_response_inner import *
from .http_validation_error import *
from .id import *
from .margin_mode import *
from .market_type import *
from .modified import *
from .notification_model import *
from .notification_type import *
from .order_model import *
from .order_status import *
from .post_futures_action import *
from .strategy_exchange_info import *
from .strategy_model import *
from .tpsl import *
from .trading_action_type import *
from .update_notification import *
from .validation_error import *
from .validation_error_loc_inner import *
